/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: accept.c,v 1.4 2006/01/28 07:02:04 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <papi.h>
#include "common.h"

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout,
		gettext("Usage: %s destination ...\n"),
		name);
	exit(1);
}

int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	int exit_status = 0;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "E")) != EOF)
		switch (c) {
		case 'E':
			encryption = PAPI_ENCRYPT_ALWAYS;
			break;
		default:
			usage(av[0]);
		}

	if (ac == optind)
		usage(av[0]);

	for (c = optind; c < ac; c++) {
		char *printer = av[c];

		status = papiServiceCreate(&svc, printer, NULL, NULL,
					cli_auth_callback, encryption, NULL);
		if (status != PAPI_OK) {
			fprintf(stderr, gettext(
				"Failed to contact service for %s: %s\n"),
				printer, verbose_papi_message(svc, status));
			exit_status = 1;
		}

		status = papiPrinterResume(svc, printer);
		if (status != PAPI_OK) {
			fprintf(stderr, gettext("accept: %s: %s\n"), printer,
				verbose_papi_message(svc, status));
			exit_status = 1;
		}

		papiServiceDestroy(svc);
	}

	return (exit_status);
}
