/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: cancel.c,v 1.3 2006/01/28 07:02:04 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <papi.h>
#include "common.h"

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout, "Usage: %s [-u user] (printer|request-id ...)\n", name);
	exit(1);
}

int
main(int ac, char *av[])
{
	int exit_code = 0;
	char *user = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "Eu:")) != EOF)
		switch (c) {
		case 'E':
			encryption = PAPI_ENCRYPT_REQUIRED;
			break;
		case 'u':
			user = optarg;
			break;
		default:
			usage(av[0]);
		}

	for (c = optind; c < ac; c++) {
		papi_status_t status;
		papi_service_t svc = NULL;
		papi_job_t *jobs = NULL;
		char *printer = NULL;
		int32_t id = -1;

		(void) get_printer_id(av[c], &printer, &id);

		status = papiServiceCreate(&svc, printer, user, NULL,
					cli_auth_callback, encryption, NULL);
		if (status != PAPI_OK) {
			fprintf(stderr, gettext(
				"Failed to contact service for %s: %s\n"),
				printer, verbose_papi_message(svc, status));
			exit(1);
		}

		if (id != -1) {	/* it's a job */
			char *mesg = "cancelled";

			status = papiJobCancel(svc, printer, id);
			if (status != PAPI_OK) {
				mesg = verbose_papi_message(svc, status);
				exit_code = 1;
			}
			fprintf(stderr, "%s-%d: %s\n", printer, id, mesg);
		} else {	/* it's a printer */
			status = papiPrinterPurgeJobs(svc, printer, &jobs);
			if (status != PAPI_OK) {
				fprintf(stderr, gettext("PurgeJobs %s: %s\n"),
					printer,
					verbose_papi_message(svc, status));
				exit_code = 1;
			}

			while ((jobs != NULL) && (*jobs != NULL))
				fprintf(stderr, "%s-%d: %s\n", printer,
					papiJobGetId(*jobs++), "cancelled");

			papiJobListFree(jobs);
		}

		papiServiceDestroy(svc);
	}

	return (exit_code);
}
