/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: lpq.c,v 1.3 2006/01/28 07:02:04 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <papi.h>
#include "common.h"

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout, gettext("Usage: %s [-P printer] (user|id ...)\n"),
			name);
	exit(1);
}

static void
clear_screen()
{
	static char buf[32];

	/* quick and dirty for now, this should be fixed real soon */
	if (buf[0] == '\0') {
		FILE *fp = popen("/bin/tput clear", "r");
		if (fp != NULL) {
			fgets(buf, sizeof (buf), fp);
			fclose(fp);
		}
	}
	printf("%s", buf);
}

int
main(int ac, char *av[])
{
	char *printer = NULL;
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	int format = 3;	/* lpq short format */
	int interval = 0;
	int num_jobs;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "EP:l")) != EOF)
		switch (c) {
		case 'E':
			encryption = PAPI_ENCRYPT_REQUIRED;
			break;
		case 'P':
			printer = optarg;
			break;
		case 'l':
			format = 4;	/* lpq long format */
			break;
		default:
			usage(av[0]);
		}

	if ((optind < ac) && (av[optind][0] == '+'))
		interval = atoi(av[optind++]);

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = DEFAULT_DEST;

	status = papiServiceCreate(&svc, printer, NULL, NULL, cli_auth_callback,
					encryption, NULL);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), printer,
			verbose_papi_message(svc, status));
		papiServiceDestroy(svc);
		exit(1);
	}

	do {
		if (interval != 0)
			clear_screen();

		num_jobs = berkeley_queue_report(svc, stdout, printer, format,
					ac - optind, &av[optind]);

		if ((interval != 0) && (num_jobs > 0))
			sleep(interval);
	} while ((interval > 0) && (num_jobs > 0));

	papiServiceDestroy(svc);

	return (0);
}
