/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: get-jobs.c,v 1.3 2006/02/01 20:03:16 njacobs Exp $"

#include <stdio.h>
#include <papi.h>
#include <ipp.h>
#include <ipp-listener.h>

papi_status_t
ipp_get_jobs(papi_service_t svc, papi_attribute_t **request,
		papi_attribute_t ***response)
{
	papi_status_t status;
	papi_job_t *j = NULL;
	papi_attribute_t **operational = NULL;

	char **req_attrs = NULL;
	char *queue = NULL;
	int limit = 0;
	char my_jobs = PAPI_FALSE;
	char *which;
	int type = 0;

	/* Get operational attributes from the request */
	(void) papiAttributeListGetCollection(request, NULL,
				"operational-attributes-group", &operational);

	/*
	 * The operational-attributes-group must contain:
	 *	printer-uri
	 */
	get_printer_id(operational, &queue, NULL);
	if (queue == NULL) {
		ipp_set_status(response, status, "printer-uri: %s",
			papiStatusString(status));
		return (PAPI_BAD_REQUEST);
	}

	/*
	 * The operational-attributes-group may contain:
	 *	limit
	 *	requested-attributes
	 *	which-jobs
	 *	my-jobs
	 */
	(void) papiAttributeListGetString(operational, NULL,
				"which-jobs", &which);
	(void) papiAttributeListGetBoolean(operational, NULL,
				"my-jobs", &my_jobs);
	(void) papiAttributeListGetInteger(operational, NULL, "limit", &limit);
	get_string_list(operational, "requested-attributes", &req_attrs);

	status = papiPrinterListJobs(svc, queue, req_attrs, type, limit, &j);
	if (status != PAPI_OK) {
		ipp_set_status(response, status, "query jobs: %s",
				ipp_svc_status_mesg(svc, status));
		return (status);
	}

	/* add any job's attributes to the response in job-attribute-groups */
	if (j != NULL) {
		int i;

		for (i = 0; j[i] != NULL; i++)
			papi_to_ipp_job_group(response, request,
					PAPI_ATTR_APPEND, j[i]);
		papiJobListFree(j);
	}

	return (status);
}
