/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: restart-job.c,v 1.3 2006/01/28 07:02:04 njacobs Exp $"

#include <stdio.h>
#include <papi.h>
#include <ipp.h>
#include <ipp-listener.h>

papi_status_t
ipp_restart_job(papi_service_t svc, papi_attribute_t **request,
		papi_attribute_t ***response)
{
	papi_status_t status;
	papi_attribute_t **operational = NULL;

	char *message = NULL;
	char *hold_until = NULL;
	char *queue = NULL;
	int id = -1;

	/* Get operational attributes from the request */
	(void) papiAttributeListGetCollection(request, NULL,
				"operational-attributes-group", &operational);

	/*
	 * the operational-attributes-group must contain:
	 *	job-uri (or printer-uri/job-id)
	 */
	get_printer_id(operational, &queue, &id);
	if (id < 0) {
		ipp_set_status(response, PAPI_BAD_REQUEST,
				"missing job-uri or job-id");
		return (PAPI_BAD_REQUEST);
	} else if (queue == NULL) {
		ipp_set_status(response, PAPI_BAD_REQUEST,
				"missing printer-uri or job-uri");
		return (PAPI_BAD_REQUEST);
	}

	/*
	 * the operational-attributes-group may contain:
	 *	message
	 *	job-hold-until
	 */
	(void) papiAttributeListGetString(operational, NULL,
				"job-hold-until", &hold_until);
	(void) papiAttributeListGetString(operational, NULL,
				"message", &message);

	if ((status = papiJobRestart(svc, queue, id)) != PAPI_OK) {
		ipp_set_status(response, status,
				"restart failed: %s-%d: %s",
				(queue ? queue : "(null)"), id,
				ipp_svc_status_mesg(svc, status));
	} else if ((message != NULL) || (hold_until != NULL)) {
		/* add unsupported attribute group */
		papi_attribute_t **unsupported = NULL;

		if (message != NULL)
			(void) papiAttributeListAddValue(&unsupported,
				PAPI_ATTR_EXCL, "message",
				PAPI_COLLECTION, NULL);
		if (hold_until != NULL)
			(void) papiAttributeListAddValue(&unsupported,
				PAPI_ATTR_EXCL, "hold-until",
				PAPI_COLLECTION, NULL);
		(void) papiAttributeListAddCollection(response,
			PAPI_ATTR_REPLACE, "unsupported-attributes-group",
			unsupported);
		papiAttributeListFree(unsupported);

		status = PAPI_OK_SUBST;
		ipp_set_status(response, status,
			"unsupported attribute in request");
	}

	return (status);
}
