/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _NIS_NSS_NIS_H
#define _NIS_NSS_NIS_H	1

#pragma ident	"$Id: nss-emulation.h,v 1.1 2005/02/26 06:58:42 njacobs Exp $"

#include <rpcsvc/ypclnt.h>

typedef enum {
  NSS_TRYAGAIN = -2,
  NSS_UNAVAIL,
  NSS_NOTFOUND,
  NSS_SUCCESS,
  NSS_RETURN
} nss_status_t;


/* Convert YP error number to NSS error number.  */
static nss_status_t yperr2nss_tab[] =
{
  [YPERR_SUCCESS] = NSS_SUCCESS,
  [YPERR_BADARGS] = NSS_UNAVAIL,
  [YPERR_RPC]     = NSS_UNAVAIL,
  [YPERR_DOMAIN]  = NSS_UNAVAIL,
  [YPERR_MAP]     = NSS_UNAVAIL,
  [YPERR_KEY]     = NSS_NOTFOUND,
  [YPERR_YPERR]   = NSS_UNAVAIL,
  [YPERR_RESRC]   = NSS_TRYAGAIN,
  [YPERR_NOMORE]  = NSS_NOTFOUND,
  [YPERR_PMAP]    = NSS_UNAVAIL,
  [YPERR_YPBIND]  = NSS_UNAVAIL,
  [YPERR_YPSERV]  = NSS_UNAVAIL,
  [YPERR_NODOM]   = NSS_UNAVAIL,
  [YPERR_BADDB]   = NSS_UNAVAIL,
  [YPERR_VERS]    = NSS_UNAVAIL,
  [YPERR_ACCESS]  = NSS_UNAVAIL,
  [YPERR_BUSY]    = NSS_TRYAGAIN
};
#define YPERR_COUNT (sizeof (yperr2nss_tab) / sizeof (yperr2nss_tab[0]))

static inline nss_status_t
yperr2nss (int errval)
{
  if ((unsigned int) errval > YPERR_COUNT)
    return NSS_UNAVAIL;
  return yperr2nss_tab[errval];
}

#endif /* nis/nss-nis.h */
