/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: psm.c,v 1.2 2006/02/23 19:35:34 njacobs Exp $"

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <dlfcn.h>
#include <papi_impl.h>

#ifndef	RTLD_GROUP
#define	RTLD_GROUP 0
#endif	/* RTLD_GROUP */

#ifndef	PSM_DIR
#define	PSM_DIR	"/usr/lib/print"
#endif

papi_status_t
psm_open(service_t *svc, char *scheme)
{
	papi_status_t result = PAPI_OK;
	char *path;
	char buf[BUFSIZ];

	if (scheme == NULL)
		return (PAPI_BAD_ARGUMENT);

	if (strchr(scheme, '/') == NULL) {
		snprintf(buf, sizeof (buf), PSM_DIR "/psm-%s.so", scheme);
		path = buf;
	} else
		path = scheme;

	svc->so_handle = dlopen(path, RTLD_LAZY|RTLD_LOCAL|RTLD_GROUP);
	if (svc->so_handle == NULL) {	/* failed, set the result/message */
		if ((access(path, F_OK) < 0) && (errno == ENOENT))
			result = PAPI_URI_SCHEME;
		else
			result = PAPI_NOT_POSSIBLE;
#ifdef DEBUG
		detailed_error(svc, "psm_open(%s): %s: %s", scheme, path,
				dlerror());
#endif
	}

	return (result);
}

void
psm_close(void *handle)
{
	dlclose(handle);
}

void *
psm_sym(service_t *svc, char *name)
{
	char *error = "invalid input";
	void *func = NULL;

	if ((svc != NULL) && (svc->so_handle != NULL) && (name != NULL)) {
		if ((func = dlsym(svc->so_handle, name)) == NULL)
			error = dlerror();
	}
#ifdef DEBUG
	if (func == NULL)
		detailed_error(svc, "psm_sym(%s): %s", name, error);
#endif

	return (func);
}
