/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: lpd-cancel.c,v 1.1 2005/02/26 06:58:42 njacobs Exp $"

#define __EXTENSIONS__	/* for strtok_r() */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <papi_impl.h>

papi_status_t
lpd_cancel_job(service_t *svc, int id)
{
	papi_status_t status = PAPI_INTERNAL_ERROR;
	int fd;
	char **list;
	char buf[128];	/* this should be overkill */

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	snprintf(buf, sizeof (buf), "%d", id);
	list[0] = buf;
	list[1] = NULL;

	if ((fd = lpd_open(svc, 'c', list, 3)) < 0)
		return (PAPI_INTERNAL_ERROR);

	memset(buf, 0, sizeof (buf));
	if (fdgets(buf, sizeof (buf), fd) != NULL) {
		if (buf[0] == NULL)
			status = PAPI_NOT_FOUND;
		else if (strstr(buf, "permission denied") != NULL)
			status = PAPI_NOT_AUTHORIZED;
		else if ((strstr(buf, "cancelled") != NULL) ||
			 (strstr(buf, "removed") != NULL))
			status = PAPI_OK;
	} else
		status = PAPI_NOT_FOUND;

	close(fd);

	return (status);
}

papi_status_t
lpd_purge_jobs(service_t *svc, job_t ***jobs)
{
	papi_status_t status = PAPI_INTERNAL_ERROR;
	int fd;
	char *queue;
	char buf[256];

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	if ((fd = lpd_open(svc, 'c', NULL, 3)) < 0)
		return (PAPI_INTERNAL_ERROR);

	queue = queue_name_from_uri(svc->uri);

	status = PAPI_OK;
	memset(buf, 0, sizeof (buf));
	while (fdgets(buf, sizeof (buf), fd) != NULL) {
		/* if we canceled it, add it to the list */
		if ((strstr(buf, "cancelled") != NULL) ||
		    (strstr(buf, "removed") != NULL)) {
			job_t *job;
			papi_attribute_t **attributes = NULL;
			char *ptr, *iter = NULL;
			int id;

			ptr = strtok_r(buf, ":", &iter);
			papiAttributeListAddString(&attributes, PAPI_ATTR_EXCL,
					"job-name", ptr);
			id = atoi(ptr);
			papiAttributeListAddInteger(&attributes, PAPI_ATTR_EXCL,
					"job-id", id);
			papiAttributeListAddString(&attributes, PAPI_ATTR_EXCL,
					"job-printer", queue);

			if ((job = (job_t *)calloc(1, (sizeof (*job))))
					!= NULL) {
				job->attributes = attributes;
				list_append(jobs, job);
			} else
				papiAttributeListFree(attributes);
		} else if (strstr(buf, "permission denied") != NULL)
			status = PAPI_NOT_AUTHORIZED;
	}
	close(fd);

	return (status);
}
